import 'dart:async';

import 'package:getx_extended/export.dart';

class RxQueryPageable<Query, Response> extends RxList<Response> {
  static var pageSize = 20;
  static var defaultFirstPage = 1;

  Future<List<Response>> Function(Query request, int page, int size) dataSource;
  Query query;

  RxQueryPageable({
    required this.query,
    required this.dataSource,
  }) : super() {
    reloadDataSource();
  }

  late int _currentPage;
  late bool _isEnded;
  late bool _isLoading;
  bool _isRefreshing = false;

  Future reloadDataSource({
    Query? updatedQuery,
  }) async {
    if (updatedQuery != null) {
      query = updatedQuery;
    }
    if (_isRefreshing) return;
    clear();
    _isRefreshing = true;
    _currentPage = defaultFirstPage;
    _isEnded = false;
    _isLoading = false;
    await next();
    _isRefreshing = false;
  }

  Response item(int index) {
    final half = pageSize / 2;
    if (index + half > value.length) {
      next();
    }
    return value[index];
  }

  Future next() async {
    if (_isEnded || _isLoading) return;
    _isLoading = true;
    return postFrameFuture(() async {
      try {
        final list = await dataSource(query, _currentPage, pageSize);
        addAllIf(true, list.toList());
        _currentPage += 1;
        _isEnded = pageSize > list.length;
      } finally {
        _isLoading = false;
      }
    });
  }
}

class RxPageable<Response> extends RxQueryPageable<void, Response> {
  RxPageable({
    required Future<List<Response>> Function(int page, int size) dataSource,
  }) : super(
          query: {},
          dataSource: (_, page, size) => dataSource(page, size),
        );
}
